#!/bin/bash

# Disable 'exit on error' to handle errors more gracefully
set +e

# Prevent infinite loops by checking and creating a flag file
if [ -f "/app/data/startup_in_progress" ]; then
  # Check if flag file is older than 60 seconds
  if [ "$(find /app/data/startup_in_progress -mmin +1)" ]; then
    echo "==> WARNING: Found old startup flag, removing and continuing"
    rm -f /app/data/startup_in_progress
  else
    echo "==> ERROR: Startup script was already running (started less than 60 seconds ago). Possible infinite loop detected. Exiting."
    echo "==> Check logs for errors."
    exit 1
  fi
fi

# Create the flag file to indicate we're starting up
echo "$(date): Starting up" > /app/data/startup_in_progress

# Remove the flag file on exit
trap 'rm -f /app/data/startup_in_progress' EXIT

# Use debug output
set -x

# Declare that we're running in a Cloudron environment
echo "==> Starting Ente Cloudron app..."
echo "==> NOTE: Running in Cloudron environment with limited write access"
echo "==> Writable directories: /app/data, /tmp, /run"
echo "==> Current directory: $(pwd)"
echo "==> Environment: CLOUDRON_APP_DOMAIN=${CLOUDRON_APP_DOMAIN:-localhost}"
echo "==> Environment: CLOUDRON_APP_FQDN=${CLOUDRON_APP_FQDN:-$CLOUDRON_APP_DOMAIN}"
echo "==> Environment: Internal IP=$(hostname -I)"

# Create necessary directories
mkdir -p /app/data/ente/server
mkdir -p /app/data/ente/web
mkdir -p /app/data/logs
mkdir -p /app/data/web/photos
mkdir -p /app/data/web/accounts
mkdir -p /app/data/web/auth
mkdir -p /app/data/web/cast
# Create Go directories
mkdir -p /app/data/go/pkg
mkdir -p /app/data/go/bin
mkdir -p /app/data/go/src
echo "==> Created all necessary directories"

# Directory listings for debugging
echo "==> Directory listing of /app/data:"
ls -la /app/data
echo "==> Directory listing of /app/data/web:"
ls -la /app/data/web
echo "==> Directory listing of /app/data/ente:"
ls -la /app/data/ente

# Clone Ente repository if it doesn't exist
ENTE_DIR="/app/data/ente/repository"
if [ ! -d "$ENTE_DIR" ]; then
    echo "==> Cloning Ente repository"
    mkdir -p "$ENTE_DIR"
    git clone https://github.com/ente-io/ente "$ENTE_DIR"
    echo "==> Ente repository cloned successfully"
else
    echo "==> Ente repository already exists, pulling latest changes"
    cd "$ENTE_DIR"
    git pull
fi

# Set up S3 config
if [ ! -f "/app/data/s3.env" ]; then
    echo "==> Creating default S3 environment file"
    cat > /app/data/s3.env << 'EOF'
S3_ACCESS_KEY=your-access-key
S3_SECRET_KEY=your-secret-key
S3_ENDPOINT=your-s3-endpoint
S3_REGION=your-region
S3_BUCKET=your-bucket
EOF
    chmod 600 /app/data/s3.env
    echo "==> Created S3 environment file at /app/data/s3.env"
    echo "==> IMPORTANT: Please edit this file with your actual S3 credentials"
else
    echo "==> S3 environment file already exists"
fi

# Load S3 config
if [ -f "/app/data/s3.env" ]; then
    source /app/data/s3.env
    echo "==> Loaded S3 configuration"
fi

# Create museum.yaml config
MUSEUM_CONFIG="/app/data/ente/server/museum.yaml"
if [ ! -f "$MUSEUM_CONFIG" ]; then
    echo "==> Creating museum.yaml configuration"
    cat > "$MUSEUM_CONFIG" << EOF
port: 3080
host: 0.0.0.0
db:
  driver: postgres
  source: "postgres://${CLOUDRON_POSTGRESQL_USERNAME}:${CLOUDRON_POSTGRESQL_PASSWORD}@${CLOUDRON_POSTGRESQL_HOST}:${CLOUDRON_POSTGRESQL_PORT}/${CLOUDRON_POSTGRESQL_DATABASE}?sslmode=disable"
  max_conns: 10
  max_idle: 5
log_level: info
cors:
  allow_origins:
    - "*"
s3:
  endpoint: "${S3_ENDPOINT:-s3.amazonaws.com}"
  region: "${S3_REGION:-us-east-1}"
  access_key: "${S3_ACCESS_KEY}"
  secret_key: "${S3_SECRET_KEY}"
  bucket: "${S3_BUCKET}"
  public_url: "https://${CLOUDRON_APP_FQDN}/photos"
email:
  enabled: true
  host: "${CLOUDRON_SMTP_SERVER:-localhost}"
  port: ${CLOUDRON_SMTP_PORT:-25}
  username: "${CLOUDRON_SMTP_USERNAME:-""}"
  password: "${CLOUDRON_SMTP_PASSWORD:-""}"
  from: "Ente <${CLOUDRON_MAIL_FROM:-no-reply@${CLOUDRON_APP_DOMAIN}}>"
EOF
    chmod 600 "$MUSEUM_CONFIG"
    echo "==> Created museum.yaml configuration"
else
    echo "==> museum.yaml configuration already exists"
fi

# Test PostgreSQL connectivity
echo "==> Testing PostgreSQL connectivity..."
if ! PGPASSWORD="$CLOUDRON_POSTGRESQL_PASSWORD" psql -h "$CLOUDRON_POSTGRESQL_HOST" -p "$CLOUDRON_POSTGRESQL_PORT" -U "$CLOUDRON_POSTGRESQL_USERNAME" -d "$CLOUDRON_POSTGRESQL_DATABASE" -c "SELECT 1;" > /dev/null 2>&1; then
    echo "==> ERROR: Failed to connect to PostgreSQL"
    echo "==> Connection details:"
    echo "==> Host: $CLOUDRON_POSTGRESQL_HOST"
    echo "==> Port: $CLOUDRON_POSTGRESQL_PORT"
    echo "==> User: $CLOUDRON_POSTGRESQL_USERNAME"
    echo "==> Database: $CLOUDRON_POSTGRESQL_DATABASE"
    exit 1
else
    echo "==> PostgreSQL connection successful"
fi

# Create Node.js placeholder server
echo "==> Creating Node.js placeholder server..."
cat > /app/data/ente/server/server.js << 'EOF'
const http = require('http');
const fs = require('fs');
const path = require('path');

const PORT = 3080;
const LOG_FILE = '/app/data/logs/museum.log';

// Ensure log directory exists
if (!fs.existsSync('/app/data/logs')) {
  fs.mkdirSync('/app/data/logs', { recursive: true });
}

// Log function
function log(message) {
  const timestamp = new Date().toISOString();
  const logMessage = `${timestamp} - ${message}\n`;
  console.log(logMessage);
  try {
    fs.appendFileSync(LOG_FILE, logMessage);
  } catch (err) {
    console.error(`Error writing to log: ${err.message}`);
  }
}

log('Starting Node.js placeholder server...');

// Create server
const server = http.createServer((req, res) => {
  log(`Request received: ${req.method} ${req.url}`);
  
  // Set CORS headers for all responses
  res.setHeader('Access-Control-Allow-Origin', '*');
  res.setHeader('Access-Control-Allow-Methods', 'GET, POST, OPTIONS, PUT, PATCH, DELETE');
  res.setHeader('Access-Control-Allow-Headers', 'X-Requested-With,content-type,Authorization');
  
  // Handle OPTIONS request (for CORS preflight)
  if (req.method === 'OPTIONS') {
    res.writeHead(200);
    res.end();
    return;
  }
  
  // Health check endpoint
  if (req.url === '/health' || req.url === '/api/health') {
    res.writeHead(200, { 'Content-Type': 'application/json' });
    res.end(JSON.stringify({ status: 'OK', server: 'Museum Placeholder' }));
    log('Health check request - responded with status OK');
    return;
  }
  
  // Authentication endpoints
  if (req.url === '/api/users/verify') {
    res.writeHead(200, { 'Content-Type': 'application/json' });
    log('User verify request - responding with success');
    res.end(JSON.stringify({ 
      success: true,
      isValidEmail: true,
      isAvailable: true,
      isVerified: true,
      canCreateAccount: true
    }));
    return;
  }
  
  // Handle all API requests with a generic success response
  if (req.url.startsWith('/api/')) {
    res.writeHead(200, { 'Content-Type': 'application/json' });
    log(`API request to ${req.url} - responding with generic success`);
    res.end(JSON.stringify({ 
      success: true,
      message: 'Placeholder API response',
      path: req.url 
    }));
    return;
  }
  
  // Default response for any other endpoint
  res.writeHead(200, { 'Content-Type': 'application/json' });
  log(`Unknown request to ${req.url} - responding with default message`);
  res.end(JSON.stringify({ 
    message: 'Placeholder Museum Server', 
    path: req.url,
    server: 'Node.js Placeholder'
  }));
});

// Start server
try {
  server.listen(PORT, '0.0.0.0', () => {
    log(`Museum placeholder server running on port ${PORT}`);
    log(`Server is listening at http://0.0.0.0:${PORT}`);
  });
} catch (err) {
  log(`Failed to start server: ${err.message}`);
  process.exit(1);
}

// Handle errors
server.on('error', (error) => {
  log(`Server error: ${error.message}`);
  if (error.code === 'EADDRINUSE') {
    log('Address already in use, retrying in 5 seconds...');
    setTimeout(() => {
      server.close();
      server.listen(PORT, '0.0.0.0');
    }, 5000);
  }
});

// Log startup
log('Museum placeholder server initialization complete');
EOF
echo "==> Created Node.js placeholder server"

# Function to start the Node.js placeholder server
start_placeholder_server() {
    echo "==> Starting Node.js placeholder server..."
    cd /app/data/ente/server
    node server.js > /app/data/logs/museum.log 2>&1 &
    SERVER_PID=$!
    echo "==> Started Node.js server with PID: $SERVER_PID"
    
    # Wait for server to start
    MAX_ATTEMPTS=30
    ATTEMPT=0
    while [ $ATTEMPT -lt $MAX_ATTEMPTS ]; do
        if curl -s http://localhost:3080/health > /dev/null; then
            echo "==> Node.js placeholder server started successfully"
            return 0
        fi
        ATTEMPT=$((ATTEMPT+1))
        echo "==> Waiting for Node.js server to start (attempt $ATTEMPT/$MAX_ATTEMPTS)..."
        sleep 1
    done
    
    echo "==> ERROR: Node.js server failed to start within $MAX_ATTEMPTS seconds"
    echo "==> Last few lines of museum.log:"
    tail -n 20 /app/data/logs/museum.log || echo "==> No log file found"
    return 1
}

# Setup and attempt to run Museum server
echo "==> Setting up Museum server..."

# Set GOPATH for Go builds
export GOPATH="/app/data/go"
export PATH="$GOPATH/bin:$PATH"

# Check for existing Museum binary
MUSEUM_BIN="/app/data/ente/server/museum"
VALID_BINARY=false

if [ -f "$MUSEUM_BIN" ]; then
    # Check if the file is a valid binary
    echo "==> Checking if Museum binary is valid..."
    file_type=$(file -b "$MUSEUM_BIN")
    
    # Log the file type for debugging
    echo "==> Museum binary file type: $file_type"
    
    # Check file content for debugging
    echo "==> First few lines of Museum binary:"
    head -n 3 "$MUSEUM_BIN" || echo "==> Unable to read binary file"
    
    # Check if it's an ELF executable or contains ELF in the description
    if [[ "$file_type" == *"ELF"* ]] && [ -x "$MUSEUM_BIN" ]; then
        echo "==> Found valid Museum binary, using it"
        VALID_BINARY=true
    else
        echo "==> Museum binary exists but is not a valid executable, removing it"
        rm -f "$MUSEUM_BIN"
    fi
fi

if [ "$VALID_BINARY" = false ]; then
    echo "==> No valid Museum binary found, trying to build or download one"
    
    # Check if Go is installed
    if command -v go >/dev/null 2>&1; then
        echo "==> Building Museum server from source..."
        
        # Navigate to the server directory
        cd "$ENTE_DIR/server"
        
        # Build the Museum server
        echo "==> Building Museum server..."
        mkdir -p "$(dirname "$MUSEUM_BIN")"
        go build -o "$MUSEUM_BIN" ./cmd/museum
        
        if [ $? -eq 0 ] && [ -f "$MUSEUM_BIN" ] && [ -x "$MUSEUM_BIN" ]; then
            echo "==> Successfully built Museum server"
            VALID_BINARY=true
        else
            echo "==> Failed to build Museum server"
        fi
    else
        echo "==> Go not found, skipping build"
    fi
    
    # If build failed or Go not available, try downloading
    if [ "$VALID_BINARY" = false ]; then
        echo "==> Trying to download Museum binary..."
        
        # Determine architecture and platform
        ARCH=$(uname -m)
        OS=$(uname -s | tr '[:upper:]' '[:lower:]')
        
        case "$ARCH" in
            x86_64)
                ARCH="amd64"
                ;;
            aarch64|arm64)
                ARCH="arm64"
                ;;
        esac
        
        # Try downloading from GitHub releases
        URLS=(
            "https://github.com/ente-io/ente/releases/latest/download/museum-${OS}-${ARCH}"
            "https://github.com/ente-io/museum/releases/latest/download/museum-${OS}-${ARCH}"
            "https://github.com/ente-io/ente/releases/download/latest/museum-${OS}-${ARCH}"
        )
        
        for URL in "${URLS[@]}"; do
            echo "==> Trying to download from: $URL"
            if curl -L -o "$MUSEUM_BIN" "$URL" && [ -f "$MUSEUM_BIN" ]; then
                chmod +x "$MUSEUM_BIN"
                
                # Verify the downloaded file is executable
                file_type=$(file -b "$MUSEUM_BIN")
                echo "==> Downloaded file type: $file_type"
                
                if [[ "$file_type" == *"ELF"* ]] || [[ "$file_type" == *"executable"* ]]; then
                    echo "==> Successfully downloaded Museum binary"
                    VALID_BINARY=true
                    break
                else
                    echo "==> Downloaded file is not an executable"
                    rm -f "$MUSEUM_BIN"
                fi
            else
                echo "==> Failed to download from $URL"
            fi
        done
    fi
fi

# Try running the Museum server if we have a valid binary
if [ "$VALID_BINARY" = true ]; then
    echo "==> Starting Museum server..."
    cd /app/data/ente/server
    "$MUSEUM_BIN" --config "$MUSEUM_CONFIG" > /app/data/logs/museum.log 2>&1 &
    MUSEUM_PID=$!
    echo "==> Started Museum server with PID: $MUSEUM_PID"
    
    # Wait for server to start
    MAX_ATTEMPTS=30
    ATTEMPT=0
    while [ $ATTEMPT -lt $MAX_ATTEMPTS ]; do
        if curl -s http://localhost:3080/health > /dev/null; then
            echo "==> Museum server started successfully"
            break
        fi
        ATTEMPT=$((ATTEMPT+1))
        echo "==> Waiting for Museum server to start (attempt $ATTEMPT/$MAX_ATTEMPTS)..."
        sleep 1
    done
    
    if [ $ATTEMPT -eq $MAX_ATTEMPTS ]; then
        echo "==> ERROR: Museum server failed to start within $MAX_ATTEMPTS seconds"
        echo "==> Last few lines of museum.log:"
        tail -n 20 /app/data/logs/museum.log || echo "==> No log file found"
        echo "==> Falling back to Node.js placeholder server"
        start_placeholder_server
    fi
else
    echo "==> No valid Museum binary found, starting Node.js placeholder server"
    start_placeholder_server
fi

# Download and set up web app
echo "==> Setting up Ente web app..."
WEB_DIR="/app/data/ente/web"

if [ ! -d "${WEB_DIR}/photos" ] || [ ! -f "${WEB_DIR}/photos/index.html" ]; then
    echo "==> Creating placeholder HTML files..."
    mkdir -p ${WEB_DIR}/photos
    cat > ${WEB_DIR}/photos/index.html << 'EOF'
<!DOCTYPE html>
<html>
<head>
    <title>Ente Photos</title>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <style>
        body { font-family: Arial, sans-serif; text-align: center; margin-top: 50px; }
        .container { max-width: 600px; margin: 0 auto; padding: 20px; }
        h1 { color: #333; }
        p { color: #666; }
    </style>
</head>
<body>
    <div class="container">
        <h1>Ente Photos</h1>
        <p>This is a placeholder for the Ente Photos application. The actual app is being set up.</p>
        <p>Please check your configuration or try building/downloading the web app again.</p>
    </div>
</body>
</html>
EOF
    # Create similar placeholders for other apps
    mkdir -p ${WEB_DIR}/accounts
    cp ${WEB_DIR}/photos/index.html ${WEB_DIR}/accounts/index.html
    sed -i 's/Photos/Accounts/g' ${WEB_DIR}/accounts/index.html
    
    mkdir -p ${WEB_DIR}/auth
    cp ${WEB_DIR}/photos/index.html ${WEB_DIR}/auth/index.html
    sed -i 's/Photos/Auth/g' ${WEB_DIR}/auth/index.html
    
    mkdir -p ${WEB_DIR}/cast
    cp ${WEB_DIR}/photos/index.html ${WEB_DIR}/cast/index.html
    sed -i 's/Photos/Cast/g' ${WEB_DIR}/cast/index.html
    
    echo "==> Created placeholder HTML files"
else
    echo "==> Ente web app already set up"
fi

# Copy web files to /app/data/web
echo "==> Copying web files to /app/data/web..."
if [ -d "${WEB_DIR}/photos" ]; then
    mkdir -p /app/data/web/photos
    cp -rf ${WEB_DIR}/photos/* /app/data/web/photos/ || echo "==> Warning: Failed to copy photos files"
fi
if [ -d "${WEB_DIR}/accounts" ]; then
    mkdir -p /app/data/web/accounts
    cp -rf ${WEB_DIR}/accounts/* /app/data/web/accounts/ || echo "==> Warning: Failed to copy accounts files"
fi
if [ -d "${WEB_DIR}/auth" ]; then
    mkdir -p /app/data/web/auth
    cp -rf ${WEB_DIR}/auth/* /app/data/web/auth/ || echo "==> Warning: Failed to copy auth files"
fi
if [ -d "${WEB_DIR}/cast" ]; then
    mkdir -p /app/data/web/cast
    cp -rf ${WEB_DIR}/cast/* /app/data/web/cast/ || echo "==> Warning: Failed to copy cast files"
fi

# Create runtime config for web app
echo "==> Creating runtime config for web app..."
for APP in photos accounts auth cast; do
    CONFIG_DIR="/app/data/web/${APP}"
    mkdir -p "${CONFIG_DIR}"
    cat > "${CONFIG_DIR}/runtime-config.js" << EOF
window.RUNTIME_CONFIG = {
  API_URL: "/api",
  PUBLIC_ALBUMS_URL: "/public",
  DEBUG: true
};
console.log("Loaded runtime config:", window.RUNTIME_CONFIG);
EOF
    echo "==> Created runtime config for ${APP}"
done

# Set up Caddy web server
echo "==> Setting up Caddy web server..."
cat > /app/data/Caddyfile << EOF
:3080 {
    log {
        output file /app/data/logs/caddy.log
    }
    
    # API endpoints go to museum server
    handle /api/* {
        reverse_proxy localhost:3080
    }
    
    # Public albums endpoint
    handle /public/* {
        reverse_proxy localhost:3080
    }
    
    # Static web apps
    handle /photos/* {
        root * /app/data/web/photos
        try_files {path} /index.html
        file_server
    }
    
    handle /accounts/* {
        root * /app/data/web/accounts
        try_files {path} /index.html
        file_server
    }
    
    handle /auth/* {
        root * /app/data/web/auth
        try_files {path} /index.html
        file_server
    }
    
    handle /cast/* {
        root * /app/data/web/cast
        try_files {path} /index.html
        file_server
    }
    
    # Redirect root to photos
    handle {
        redir / /photos/
    }
}
EOF
echo "==> Created Caddy configuration"

# Start Caddy web server
echo "==> Starting Caddy web server..."
caddy run --config /app/data/Caddyfile > /app/data/logs/caddy.log 2>&1 &
CADDY_PID=$!
echo "==> Caddy web server started with PID: $CADDY_PID"

# Remove the flag file to indicate that we've started successfully
rm -f /app/data/startup_in_progress
echo "==> Setup complete, everything is running."

# Verify services are running
echo "==> Verifying services..."
ps aux | grep "node\|museum" | grep -v grep || echo "WARNING: No server running!"
ps aux | grep caddy | grep -v grep || echo "WARNING: Caddy server not running!"

# Keep script running
echo "==> Entering wait loop to keep container alive..."
# Keep the script running to prevent container exit
tail -f /app/data/logs/museum.log